const express = require('express');
const bodyParser = require('body-parser');
const sequelize = require('./config/database');
const Team = require('./models/Team');
const Match = require('./models/Match');
const path = require('path');
const { Op } = require('sequelize');

const app = express();
app.use(bodyParser.json());

// Serve static files from the "public" directory
app.use(express.static(path.join(__dirname, 'public')));

// Reset the database and recreate the tables
app.get('/mr-padel-reset-got', async (req, res) => {
    try {
      // Force sync the database: this will drop existing tables and recreate them
      await sequelize.sync({ force: true });
  
      // Optionally, you can reseed the database with the initial team data
      await Team.bulkCreate([
        { name: 'Unreal Unity', pool: 'A' },
        { name: 'Murshads', pool: 'A' },
        { name: 'No One', pool: 'A' },
        { name: 'Megatron', pool: 'A' },
        { name: 'Team Kazuya', pool: 'A' },
        { name: 'Team IPPO', pool: 'A' },

        { name: 'Team Khatra', pool: 'B' },
        { name: 'Tehelka', pool: 'B' },
        { name: 'Surprise', pool: 'B' },
        { name: 'Team H&M', pool: 'B' },
        { name: 'AA Tigers', pool: 'B' },
        { name: 'Tax Collectors', pool: 'B' }
      ]);
  
      res.status(200).send('Database has been reset and tables have been recreated.');
    } catch (err) {
      console.error('Error resetting the database:', err);
      res.status(500).send('Error resetting the database.');
    }
  });

  // Endpoint to get teams by pool
app.get('/teams/:pool', async (req, res) => {
    const pool = req.params.pool;
  
    // Fetch teams based on the pool (A or B)
    const teams = await Team.findAll({ where: { pool } });
    
    if (teams.length === 0) {
      return res.status(404).json({ error: `No teams found for Pool ${pool}` });
    }
  
    res.status(200).json(teams);
  });
  


// Add match API
app.post('/match', async (req, res) => {
    try {
      const { team1, team2, score1, score2, pool } = req.body;
  
      // Check if a match between these teams (in any order) has already been played
      const existingMatch = await Match.findOne({
        where: {
          pool: pool,
          [Op.or]: [
            { team1: team1, team2: team2 },
            { team1: team2, team2: team1 }
          ]
        }
      });
  
      if (existingMatch) {
        return res.status(400).json({ error: 'A match between these teams has already been played.' });
      }
  
      const winner = score1 > score2 ? team1 : team2;
  
      // Record the match
      await Match.create({ team1, team2, score1, score2, winner, pool });
  
      // Update Team Stats
      const teamOne = await Team.findOne({ where: { name: team1 } });
      const teamTwo = await Team.findOne({ where: { name: team2 } });
  
      teamOne.gamesPlayed += 1;
      teamTwo.gamesPlayed += 1;
  
      // Update score difference
      teamOne.scoreDifference += (score1 - score2);
      teamTwo.scoreDifference += (score2 - score1);
  
      // Update win/loss and points
      if (winner === team1) {
        teamOne.gamesWon += 1;
        teamTwo.gamesLost += 1;
        teamOne.points += 3; // Winning team gets 3 points
      } else {
        teamTwo.gamesWon += 1;
        teamOne.gamesLost += 1;
        teamTwo.points += 3; // Winning team gets 3 points
      }
  
      // Save updated stats
      await teamOne.save();
      await teamTwo.save();
  
      res.status(201).json({ message: 'Match recorded successfully.' });
    } catch (error) {
      console.error('Error posting the match:', error);
      res.status(500).json({ error: 'An error occurred while processing the match.' });
    }
  });
  
  
  
  

  app.get('/standings/:pool', async (req, res) => {
    try {
      const pool = req.params.pool;
      const teams = await Team.findAll({ where: { pool } });
  
      // Sort by points, and for teams with the same points, sort by score difference
      const standings = teams.map(team => ({
        name: team.name,
        gamesPlayed: team.gamesPlayed || 0,
        gamesWon: team.gamesWon || 0,
        gamesLost: team.gamesLost || 0,
        points: team.points || 0,
        scoreDifference: team.scoreDifference || 0
      })).sort((a, b) => {
        if (b.points === a.points) {
          return b.scoreDifference - a.scoreDifference; // Sort by score difference if points are equal
        }
        return b.points - a.points; // Sort by points
      });
  
      res.status(200).send(standings);
    } catch (error) {
      console.error('Error fetching standings:', error);
      res.status(500).json({ error: 'An error occurred while fetching the standings.' });
    }
  });


  app.get('/matches', async (req, res) => {
    try {
      const matches = await Match.findAll({ order: [['createdAt', 'DESC']] }); // Order by most recent
      res.status(200).json(matches);
    } catch (error) {
      console.error('Error fetching matches:', error);
      res.status(500).json({ error: 'An error occurred while fetching matches.' });
    }
  });
  
  
  
  

const PORT = process.env.PORT || 3000;
app.listen(PORT, () => {
  console.log(`Server is running on port ${PORT}`);
});
